globals [ patches-in-block  instructions am-I-set-up? key-exists? abort-create-key?
          all-order-match? all-no-order-match?  count-steps    wait-a-moment?
          dice-list  no-order-choices   no-order-success?  samples-counter 
          set-sum sum-dice sum-dice-list 
          count-key-sum-dice-successes   key-sum-dice-successes-per-sample-list         
          count-order-successes          #order-successes-per-sample-list     mean-order-per-sample   
          count-no-order-successes       #no-order-successes-per-sample-list  mean-no-order-per-sample ]
breeds [frames dice]
dice-own [ my-shape  my-set-number  my-current-number]

to startup
    initialize
    ask patches [set pcolor white]
    set instructions "Hi! Please press Setup."  
end

to initialize
    ca 
    set dice-list ["one" "two" "three" "four" "five" "six"] 
    set all-order-match? false   set all-no-order-match?  false   
    set key-exists? false   set am-I-set-up? false       
    set no-order-choices []   set key-sum-dice-successes-per-sample-list []
    set #order-successes-per-sample-list []   set #no-order-successes-per-sample-list []  
    set sum-dice-list [0 0 0 0 0 0 0 0 0 0 0]  ;; The sum of two dice ranges from 2 (first list item) to 12 (last list item)
    set display? true set bars? true   set wait-a-moment? false    set abort-create-key? false   
end

;;--------------------------------------------------------------------------------------------------------
;;--------------------------------------------------------------------------------------------------------

to setup
    locals [running?]
    set running? count-steps   wait .1   
    if running? != count-steps 
      [set wait-a-moment? true  wait .5 
       set instructions "First, unpress 'search-key.'" 
       set wait-a-moment? false  stop]
    display 
    initialize  
    create-block
    create-frames   
    set instructions "OK, you have created your dice block.  Now press 'create-key' to set their values."
    set am-I-set-up? true                      
end

to create-block
    set patches-in-block patches with [pycor = 0 and pxcor > -1]
    ask patches-in-block [set pcolor green]
    ask patches with [pcolor != green] [set pcolor white]
end 

to create-frames
    set-default-shape frames "frame"
    ask patches-in-block
    [
      sprout 1 
      [ set breed frames
        set color black ]  
    ]
end

to create-key
    set display? true
    if abort-create-key? [stop]
    if not am-I-set-up? [alert-setup stop]
    set instructions "1) Mouse-click on the patches to create your key. " + 
                     " Repeated clicks will change appearance. 2) UNPRESS the button. 3) Press 'hide-key'."
    assign-face
    set no-order-choices [] 
    
    ;; no-order-choice is a list of the key dice's face-values;
    ;; This list will serve us later in the "check-no-order" procedure, 
    ;; where we compare between an event of two face-values, 
    ;; e.g,. [3; 4], and the key values. It is called "no-order-choice" 
    ;; becasue in the comparison we will accept as a match both the order of 
    ;; the number positions [3; 4] as well as the reverse order, [4; 3]
    set no-order-choices values-from dice [ 1 +  position shape dice-list ] 
    set key-exists? true 
end

;;in the create-key procedure. For every mouse-click on a die face, 
;;the procedure identifies the die's current face value, moves one step 
;;forward in the list of 6 face-value options and assigns this new value to the current die
to assign-face
    if mouse-down?
    [
      ask patches-in-block with [(pxcor = round mouse-xcor) and (pycor = round mouse-ycor) ]
      [
            ifelse count dice-here < 1
             [
              sprout 1 
                [
                 set breed dice
                 set color black
                 set shape "one"               
                ]
             ]
             [
               ;; see NETLOGO FEATURE in the Information tab
               ask dice-here [set shape ( item ((1 + position shape dice-list) mod 6) dice-list ) ]
             ]      
          ]                     
         wait .3      ;; the wait gives the user a chance to lift their finger of the mouse/trackpad             
      ]   
    ask dice [  set my-shape shape
                set my-set-number (1 +  position shape dice-list ) ]
    set set-sum sum values-from dice [my-set-number]
end

to hide-key    
    if not am-I-set-up? or not key-exists? [alert-setup stop]
    set abort-create-key? true
    set instructions "You are now ready for the stochastic key hacking.  If you give up, press 'reveal-key.'" + 
                     " Now press 'search-key.'"  
    ask dice [ht]
    ask patches-in-block [set pcolor yellow]   
end

to search-key
    locals [calc-order-help calc-no-order-help]
    if samples-counter = total-samples [stop]
    if sample-size = 0 [user-message "Please set sample-size to a minimum larger than 0." stop]
    if not am-I-set-up? or not key-exists? [alert-setup stop]
    set abort-create-key? true
    if count dice < 2 [user-message "Hey you - yes, YOU!!! \n What about, like," + 
                                     " setting up a pair of dice? \n Press 'OK' then 'setup' and start again." stop]
    ifelse wait-a-moment? 
      [wait 4 display]
      [set instructions "Patches are repeatedly and simultaneously choosing one of the choices randomly and recording " +
                     " the number of times the secret key is matched."]
    if count dice = 0 [set instructions "Please first set up your dice." stop]    
    set all-order-match? true
    set all-no-order-match? true         
    ask dice [guess]
    check
    set count-steps count-steps + 1
    if all-order-match? 
    [
      set count-order-successes count-order-successes + 1   
      if single-success?  
      [
        display
        set calc-order-help  ( length #order-successes-per-sample-list  * sample-size ) + count-steps
        set instructions "Congratulations! You hacked the hidden key in " + calc-order-help + " steps. " + "Set up and try again." 
        wait .5
        set count-steps 0     
        set count-order-successes 0        set #order-successes-per-sample-list []
        stop      
      ]
    ]
    if all-no-order-match? 
    [
      set count-no-order-successes count-no-order-successes + 1 
      if single-success? 
      [
        display
        set calc-no-order-help  ( length #no-order-successes-per-sample-list  * sample-size ) + count-steps
        set instructions "Congratulations! You hacked the hidden key in " + calc-no-order-help + " steps. " + "Set up and try again." 
        wait .5
        set count-steps 0     
        set count-no-order-successes 0     set #no-order-successes-per-sample-list []
        stop      
      ]
    ]  
    ifelse display? [display] [no-display]
    if show-sums-histogram? [ plot-sum-dice ]
    if count-steps = sample-size [plot-sample]
end

to guess  ;; dice procedure
   ;; each die chooses a face-value randomly from the dice-list 
   set shape item ( random 6 ) dice-list 
   st 
   set my-current-number (1 +  position shape dice-list )
end

to check
   ask dice [check-order]   
   check-no-order
   check-sum
end

to check-order  ;; patches-in-block procedure
   if my-current-number != my-set-number [set all-order-match? false] 
end

to check-no-order 
    ;;the face-values of the dice are compared both to the original pair [A;B] or to its reverse order [B;A]
    locals [ current-dice ]
    set current-dice (values-from dice [ my-current-number ] )   
    ifelse ( current-dice = no-order-choices ) or
           ( current-dice = reverse no-order-choices )
      [set all-no-order-match? true] 
      [set all-no-order-match? false]
end

to check-sum 
    set sum-dice sum values-from dice [my-current-number]  ;;each dice "looks up" its number; these are summed 
    set sum-dice-list ( replace-item (sum-dice - 2) ( sum-dice-list) ( ( item (sum-dice - 2) sum-dice-list ) + 1)  ) 
    if sum-dice = set-sum [ set count-key-sum-dice-successes count-key-sum-dice-successes + 1 ]
end

to reveal-key
    set abort-create-key? true wait 1    
    if not am-I-set-up? or not key-exists? [alert-setup stop]   
    set wait-a-moment? true     wait .5 
    display
    ask dice [set shape my-shape] 
    no-display
    set instructions "Here is the secret key.  To start again, press 'setup.'"
    wait 5
    set am-I-set-up? false
    set wait-a-moment? false
end

to alert-setup
   set instructions "Please work this way: Press 'setup,' and then 'create-key'"
end

to plot-sample 
    set samples-counter samples-counter + 1 
    set #order-successes-per-sample-list          fput count-order-successes           #order-successes-per-sample-list
    set #no-order-successes-per-sample-list       fput count-no-order-successes        #no-order-successes-per-sample-list
    set key-sum-dice-successes-per-sample-list fput count-key-sum-dice-successes key-sum-dice-successes-per-sample-list
    if order-matters? != "no"
    [ do-plot "Key Frequency Distribution" #order-successes-per-sample-list "order" ]    
    if ( order-matters? != "yes" )
    [ do-plot "Key Frequency Distribution" #no-order-successes-per-sample-list "no-order" ]
    do-plot "Key Sum Frequency Distribution" key-sum-dice-successes-per-sample-list "default"
end

to do-plot [ the-plot event-list current-plot-name ]
  set-current-plot the-plot
  set-current-plot-pen current-plot-name
  ifelse bars?  [set-plot-pen-mode 1] [set-plot-pen-mode 0]   ;; bug - currently the argument primitives don't function in NetLogo1.2pre5
  ifelse the-plot = "Key Sum Frequency Distribution"
    [set-plot-x-range 0 ceiling (1 + max list ( 2 * mean event-list ) ( max event-list ) ) ]
    [set-plot-x-range 0 (1 + max list ( max #order-successes-per-sample-list ) ( max #no-order-successes-per-sample-list ) ) ]
  histogram-list event-list
  set count-steps 0
  set count-order-successes 0
  set count-no-order-successes 0
  set count-key-sum-dice-successes 0
end

to plot-sum-dice
    locals [counter]
    set-current-plot "Dice Sum Frequency Distribution"
    set counter 2
    repeat 11
    [
      ppu
      plotxy counter 0
      ppd
      plotxy counter item (counter - 2) sum-dice-list
      set counter counter + 1
    ]
end

to-report ratio
    report ( mean #no-order-successes-per-sample-list / mean #order-successes-per-sample-list )
end

; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2002 by Uri Wilensky.  Updated 2002.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2002).  NetLogo Dice model.
; http://ccl.northwestern.edu/netlogo/models/Dice.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Dice
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
137
60
366
164
1
0
73.0
1
10
1
1
1

CC-WINDOW
807
65
945
296
Command Center

BUTTON
674
596
783
629
NIL
reveal-key
NIL
1
T
OBSERVER

MONITOR
80
302
145
351
+  events
count-steps
3
1

MONITOR
246
303
312
352
ORDER
count-order-successes
3
1

MONITOR
298
528
389
577
max-ORD
max #order-successes-per-sample-list
3
1

MONITOR
8
528
102
577
min-ORD
min #order-successes-per-sample-list
3
1

SWITCH
120
165
261
198
single-success?
single-success?
1
1
-1000

MONITOR
7
302
77
351
#samples
samples-counter
0
1

SWITCH
120
200
261
233
bars?
bars?
0
1
-1000

MONITOR
7
580
102
629
min-NO-ORD
min #no-order-successes-per-sample-list
3
1

MONITOR
298
580
389
629
max-NO-ORD
max #no-order-successes-per-sample-list
3
1

MONITOR
312
303
392
352
NO-ORDER
count-no-order-successes
3
1

TEXTBOX
155
311
245
346
Successes in \nthis sample:

BUTTON
8
63
117
96
NIL
setup
NIL
1
T
OBSERVER

BUTTON
8
98
117
131
NIL
create-key
T
1
T
OBSERVER

BUTTON
8
132
117
165
NIL
hide-key
NIL
1
T
OBSERVER

BUTTON
8
166
117
267
NIL
search-key
T
1
T
OBSERVER

MONITOR
7
10
945
59
NIL
instructions
0
1

MONITOR
104
528
199
577
mean-ORD 
mean #order-successes-per-sample-list
2
1

PLOT
8
355
389
527
Key Frequency Distribution
successes-per-sample
frequency
0.0
16.0
0.0
50.0
true
true
PENS
"order" 1.0 1 -16777216 true
"no-order" 1.0 1 -65536 true

MONITOR
104
580
199
629
mean-NO-ORD 
mean #no-order-successes-per-sample-list
2
1

MONITOR
715
295
781
344
NIL
sum-dice
3
1

PLOT
393
64
780
242
Dice Sum Frequency Distribution
      2     3     4     5    6     7     8      9    10    11   12 
Frequency
0.0
14.0
0.0
100.0
true
false
PENS
"Dice-Sum" 1.0 0 -16777216 true

MONITOR
394
243
780
292
NIL
sum-dice-list
3
1

SWITCH
394
303
584
336
show-sums-histogram?
show-sums-histogram?
1
1
-1000

SWITCH
120
234
261
267
display?
display?
0
1
-1000

TEXTBOX
588
300
703
345
Updated constantly. It will slow down the search.

PLOT
394
355
782
527
Key Sum Frequency Distribution
Key-Sum Successes per Sample
frequency
0.0
27.0
0.0
10.0
true
false
PENS
"default" 1.0 1 -16777216 true

MONITOR
568
529
641
578
mean-sum
mean key-sum-dice-successes-per-sample-list
2
1

TEXTBOX
394
580
563
636
mean values can be thought of as percentage when the sample size is 100

MONITOR
394
529
460
578
min-sum
min key-sum-dice-successes-per-sample-list
2
1

MONITOR
715
530
782
579
max-sum
max key-sum-dice-successes-per-sample-list
3
1

MONITOR
200
554
296
603
NO-ORD / ORD
ratio
3
1

MONITOR
569
580
641
629
NIL
set-sum
3
1

SLIDER
8
268
204
301
sample-size
sample-size
0
1000
100
5
1
NIL

SLIDER
206
268
392
301
total-samples
total-samples
0
1000
500
10
1
NIL

CHOICE
262
188
392
233
order-matters?
order-matters?
"no" "yes" "both"
0

@#$#@#$#@
WHAT IS IT?
-----------
This model is a part of the ProbLab curriculum. The ProbLab Curriculum is currently under development at the CCL. For more information about the ProbLab Curriculum please refer to http://ccl.northwestern.edu/curriculum/ProbLab/.

"Dice" is a virtual laboratory for studying fundamental probabilistic aspects of a classic pair of dice, each with 6 sides. The model simulates randomly rolling dice and uses statistical analytic tools to make sense of data accumulated over numerous rolls (1000 rolls per second!). Because the model tracks so many trials, the ratio of favored to un-favored events or of events per trial rapidly converges towards the limiting values of the dice's probabilities. By examining these asymptotic proportions through numerical and graphic lenses, the user gains insight into fundamental concepts of probability. At the same time, because the analysis is after the fact and the rolls are so numerous as compared to the relatively small sample space, there is a close match between mathematical and empirical probability. One could say that "probability meets statistics on the corner of Tools and Infinity."

There are 36 different outcomes in the dice sample-space (why 36?), and 11 different sums (why 11?). If we keep rolling a pair of dice, how often will they fall on a particular key, e.g., [3;4]?  If we could track which die is which then [3;4] would not be the same as [4;3], right? -- that is, "order counts."  But usually, at least in dice games, order does not count (our "NO-ORDER" condition). What is the relation between the frequency of events under the order-counts and the no-order conditions? Is it the same for all keys? How about the sum of the dice? Mmmmm, [3;4] would be the same as [5;2] as far as the sum is concerned. Here, of course there is no question of order, but perhaps some sums are more frequent than others? Is there regularity to this? What is it? Why is it?

In "Dice," the user pre-determines how many events the model will produce and how the string of events will be parsed. Parsing is done in the form of "samples," which are just concatenated sub-strings of successive events. Standing at the statistics helm, the user must make decisions that inform the simulation: How many samples do I want? How large will each sample be? Perhaps if samples are too small and/or too few then we will not get the characteristic central-tendency values. So should one always take the biggest and most numerous samples? But what if I want results immediately and just can't afford to wait? Actually, what IS a sufficient size and number of runs anyway? Following such deliberations and hypothesizing, and by running the simulation, the user observes data growing before her eyes -- as the graphic shapes take form, she can observe the number of events in a monitor and confirm or disconfirm her hypotheses.


HOW IT WORKS
------------
The dice are a breed of agents. Each die has its original face value, e.g., 3, and an image that goes with that number, i.e. three dots. Naturally, these being classic dice, there are 6 options per die. These options are in a list called "dice-list." When the user sets these dice by clicking on them, they remember their original values. Then, the search begins. The computer randomly assigns face values and corresponding images to each die from the dice-list. There are then several checks. First, each die knows if its current value matches its original value, which it remembers. Now...

1. If the dice have switched values -- e.g., the original pair was [2;5] and the current pair is [5;2] -- we have a "NO-ORDER" match.

2. If both die have their original value then we also have an "ORDER" match, because what you see on the screen is necessarily what you set.
[Note that the ORDER matches are contained within the "NO-ORDER" matches, e.g., if the original pair was [2:5] then both [2;5] and [5;2] satisfy "NO-ORDER" but only [2; 5] satisfies the "ORDER" match.

3. There is a separate search -- the dice-sum search -- for dice that give the same total as the original pair, so for [2;5] (sum of 7), this search will accept, e.g., [4;3],[3;4], [6;1], etc. 

4. There is a search for sums of dice which runs along irrespective of the original pair.

The four searches compile their data onto their respective lists, and these data appear in monitors and are plotted in graphs. The successive events are parsed into samples (you set their length) so the number of successes ("hits") in each sample, for each of searches 1 thru 3, constitute the value of that sample.  These are the plotted values.


HOW TO USE IT
-------------
Set the dice to a key, e.g., [4:1] then command the computer to search for this key. The program will keep rolling over and over and will keep track of how many "hits" (successes) it has had, as far as the specific ordered key goes ( [4:1] ) or as far as the dice go regardless of order ( [4:1] or [1:4] ) or as both the above. Also, it will track how many hits of the sum it has had, e.g., the sum of the [4:1] key is 5, and so [3:2] and other events count for this sum. Finally, the model tracks the frequency of sums over all runs. This data is all represented in monitors and graphs. Set your key, run the model, and watch how the numbers grow and the graphs take characteristic form. Here's how you go about it step by step (These instructions appear in the "instructions" monitor):

Press 'setup' to get things started -- you will get two green squares that are placeholders for the pair of dice. Now, press 'create-key' and then click on each of these green areas -- you will get the familiar icons of dice faces. By clicking on a 'one' you will get a 'two' and by pressing on a 'two' you will get a 'three' and so on. If you press on a 'six,' you get a 'one' again. Once you have set both dice, click once again on 'create-key.' The dice you have chosen make up your "key," You can hide your key by pressing on 'hide-key.' That way, you can set the computer for another user who will not know what the key is, unless they analyze the monitors and plot windows once the model is running. 

Press 'search-key.' You don't need to search yourself -- the model will do it for you. But you will have to study the numbers in the monitors and the graphs in order to understand what is happening as the computer searches.  Of course, the idea of "searching" is just our way of asking about probabilities. Usually, people say, "there is a 1 in 6 chance that a die will fall on "5" because the die has 6 sides and unless it is weighted, it should have an equal chance of falling on each of these sides because is it a cube and cubes are symmetrical body," or something like that. In "Dice" the code gives each die side an equal chance, that is, the die-ness is built into the code. That is why we say this model simulates the rolling of dice. By talking about "searching" we are giving an ecological and playful context to all this rolling: The computer searches and monitors its successes, you need only watch and understand. It is perhaps confusing that the computer both "searches" and "monitors" because if it knows the answer (the "key") then why does it need to search at all? When you will work in the NetLogo model "Stochastic Cryptology," the idea of searching may make more sense, because the sample space can be really big in that model, to the extent that it takes the computer so long to find the key even once, that one asks "hmmm, perhaps the computer could find the key quicker if it searched more intelligently." Then, the user can tamper with the code. Of course, such tampering deviates from "pure" probability, but hey, life is complex. So we're calling it a search to prepare you for that model. OK?

If at any time you have forgotten what the key is you can press 'reveal-key.'

Observe the plot windows, beginning on the bottom left. Match between the red and black graphs and the monitors by using the "pen" legend in the plot and the names of the monitors below the graph. Connect these numerical values to the sample-size setting on the slider. Experiment with different keys and different sample-sizes and number of samples. 

buttons:
setup -- begins new experiment
create-key -- user sets the values of the dice (the frequency of which will be studied in the experiment)
hide-key  -- user effaces the faces of the dice, but this information is kept as numerical values in the program
search-key -- the program begins rolling the dice as many times as the user has set
reveal-key -- the program shows you what your original key was. If you press this while in a search, the search will stop.

switches:
single-success? -- curtails the search after the key has been matched once.
bars?  -- toggles between two graphing options: "on" is a histogram, and "off" gives a line graph.
display? -- when display? is "off" the dice in the graphics window freeze. Although the faces do not change the search continues behind the curtains and the monitoring widgets keep updating. So why is it good? Because the program has limited working-memory resources and displaying the faces takes time, too (well, very little time per change, but this certainly accumulates). If the program need not display, it can invest its resources in a quicker search. You'll probably want to use this only once you are familiar with the model. 
show-sum-histograms? -- the "on" setting causes the program to display the distribution frequency of the sum of the dice per each and every event. This, too, is time/memory consuming.

choices:
order-matters? -- toggles between options of monitoring and displaying: ("No") matches regardless of dice left-right location, e.g., [3; 4] is matched both by [3; 4] and by [4; 3]; ("Yes") only absolute matches, e.g., a [3; 4] key is matched only by a [3; 4] event; and ("both") in which both types of searches are monitored and displayed.

sliders:
sample-size
total-samples

monitors:
#samples -- how many samples have been taken up to this point in this experiment
+ events -- how many single events have occurred within the current sample (the "+" suggests that if you wish to know how many events have occurred in total since the beginning of the experiment then you must calculate #samples * sample-size + events).
ORDER -- how many events of type order-counts (key is matched in correct position) have occurred in this sample
NO-ORDER -- how many events of type no-order (key is matched in either position) have occurred in this sample
NO-ORD / ORD -- the ratio between the number of occurrence of each kind.
min-ORD, mean-ORD, max-ORD ---- the minimum, mean, and maximum values, respectively, in the order-counts experiment
min-NO-ORD, mean-NO-ORD, max-NO-ORD -- the minimum, mean, and maximum values, respectively, in the no-order experiment
sum-dice -- the sum of the dice in this event
min-sum, mean-sum, max-sum -- the minimum, mean, and maximum values, respectively, in the key-sum experiment
set-sum -- the sum of the dice you have set in the key
sum-dice-list -- this list is the cumulative frequencies of the dice sum 2 thru 12. This list is the source of the graph above it, the Dice Sum Frequency Distribution

plots:
Key Frequency Distribution -- displays the frequency of the number of successes per sample over all samples you have taken. For instance, if on the first five samples you have taken, the key was matched 3 times, 2 times, 4 times, 7 times, and 4 times, then the graph will be the same height over 2, 3, and 7, but it will be twice as higher over the 4, because 4 occurred twice.

Dice Sum Frequency Distribution -- displays the frequency of dice sums over all trials, irrespective of sample size. This graph will always produce roughly the same values for a given N number of trials.

Key Sum Frequency Distribution -- displays the frequency of the set-sum over samples. 


THINGS TO NOTICE
----------------
The histogram in the plot window on the bottom right -- the Key Sum Frequency Distribution plot -- can be thought of as an elaboration on a single column from the plot above it -- Dice Sum Frequency Distributions. This is because whereas the bottom graph shows the frequency distribution of a single sum, e.g., 8, and reflects only how many of these sum-events occurred per sample, the top graph compiles the distribution information of every single sum-event, irrespective of the sum or of sample size. So if the bottom graph clusters around a frequency that is 14% of the sample (e.g., a mean of 14 8-events out of every 100 trials) then in the above graph the number and column corresponding to the 8-events is 14% of the cumulative events. To see what the total number of events has been, you can either enter the following code -- "show sum sum-dice-list" -- in the command center, then press the RETURN key on your keyboard, or simply computer the number of samples you have taken * the sample size. To make this clearer, we can stick to simple numbers. Let us assume that you set the dice at [3:5] and have run the model over 1,000 samples of size 100 trials. Do it, then come back here. I'll wait.

Hi, welcome back. If the mean value in the bottom graph is 14.0, then in the top graph and the list below it the value corresponding to 8 is 14,000. It is as though in the bottom graph you took all the columns and put them one on top of the other and what you get is the 8-column in the top graph. This is how you can get the compiled value from the bottom graph: In the command center, enter "show sum key-sum-dice-successes-per-sample-list" and press the RETURN key on your keyboard. What did you get?


THINGS TO TRY
-------------
Run the model over a low number of samples and sample-size. See the resulting shape of the graphs. Repeat this several times and see if you got more or less the same graph. Now do this over for a medium sample size and number of samples. Finally, do this experiment for high values of sample size and number. Note how the bigger the values of sample size and number of samples, the less the shape of the graph changes. What is this shape? Have you come across it in different domains?


For certain keys, if you run the search under the "both" option of the 'order-matters?' choice, you will see only single graph in the Key Frequency Distribution plot. Try to find such a key, then try to find others. What do these keys have in common? Why do you think you observe only a single graph line? Where is the other line? How do the displays of the ORD, NO-ORD, and ORD/NO-ORD monitors behave under these various setting? [<--- that was a clue, buddy]

For other keys, the ORDER and NO-ORDER graphs will not overlap. How close are they? Using the same key repeatedly, try different sample-sizes. What happens to the graphs? If the distance between them changed, has something else remained the same about their relation? If the distance has not changed, note that the value of the x-axis has -- it has been updated automatically in the code. This was done specifically to keep the pattern of the black and red graphs unchanged amidst the change in sample size. What's going on here?


EXTENDING THE MODEL
-------------------
Actually, the Stochastic Cryptology model is an extension of this model (the truth is -- shhhhhhh!!! -- that Stochastic Cryptology was developed first, and "Dice" is its simplification). In the other model, you can roll more than just two dice. Nevertheless, a huge challenge for you is to add a 7th face to the options of this model. If you can do this, then you have truly mastered the code of "Dice."


NETLOGO FEATURES
----------------
An interesting feature of "Dice," that does not appear in many other models, is the procedure for selecting a die's face value. There is a list of six values which correspond to six shapes. To you, it is obvious that three dots means "3," but the program doesn't know this. We have to match between each face and its corresponding value. This is done in the 'assign-face' procedure, which mediates between the users clicking on a die and the values of the die, by using the code:

    ask dice-here [set shape ( item ((1 + position shape dice-list) mod 6) dice-list ) ]

So the code in the "[ ]" addresses the die in the patch that the user it clicking on. It is asking the die to set its shape to a certain shape in the dice-list. In fact, it is asking the die to re-set its shape to the shape that is located 1 place greater than the location of the die's current shape. If you look up in the "initialize" procedure, you will see the line:

    set dice-list ["one" "two" "three" "four" "five" "six"]

So if the current shape of the dies is "four" then the assign-face procedure will push it over to "five," etc. The "mod 6" means that if the shape is "six" then one up from there will be "one" again. So "mod" helps us cycle through a list. But what are these shapes? Well, images don't come from nowhere. I created these images in the "Shapes Editor" that's in the "Tools" drop-down menu. I made a die face with face value 1 and called it "one." I made a die face with face value 2 and called it "two," and so on. In the "to guess" procedure you will see the lines of code.

    set my-set-number (1 +  position shape dice-list ) 

my-set-number is the key face-value set by the user when they create the key. This line, addressing each die, connects between the die's shape and its face value. It locates the position of the shape name, e.g., "four," in the dice-list and adds 1 to that. We need to add 1 because whereas for you the position of "four" on the dice-list is simply the 4th position, for the program the position of "four" on the dice-list is "3." This is because the program calls first position the 0 position. 


CREDITS AND REFERENCES
----------------------
Thanks to Dor Abrahamson for his work on the design of this model and the ProbLab curriculum.

To refer to this model in academic publications, please use: Wilensky, U. (2002). NetLogo Dice model. http://ccl.northwestern.edu/netlogo/models/Dice. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Dice for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

five
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Circle -1 true false 182 180 41
Rectangle -16777216 true false 120 120 180 178
Circle -1 true false 76 178 41
Circle -1 true false 180 77 40
Rectangle -1 true false 200 97 206 106
Circle -1 true false 131 130 39

four
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Circle -1 true false 182 180 41
Rectangle -16777216 true false 120 120 180 178
Circle -1 true false 76 178 41
Circle -1 true false 180 77 40
Rectangle -1 true false 200 97 206 106
Circle -1 true false 131 130 39
Rectangle -16777216 true false 121 114 178 179

frame
false
14
Rectangle -16777216 true true -8 2 18 298
Rectangle -16777216 true true 1 0 300 16
Rectangle -16777216 true true 283 2 299 300
Rectangle -16777216 true true 1 285 300 299

one
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Rectangle -16777216 true false 67 62 119 123
Rectangle -16777216 true false 186 182 223 227
Rectangle -16777216 true false 177 194 198 211
Rectangle -16777216 true false 58 55 128 136
Rectangle -16777216 true false 181 191 247 239

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

sheep
false
15
Rectangle -1 true true 90 75 270 225
Circle -1 true true 15 75 150
Rectangle -16777216 true false 81 225 134 286
Rectangle -16777216 true false 180 225 238 285
Circle -16777216 true false 1 88 92

six
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Circle -1 true false 182 180 41
Rectangle -16777216 true false 120 120 180 178
Circle -1 true false 76 178 41
Circle -1 true false 180 77 40
Rectangle -1 true false 200 97 206 106
Circle -1 true false 74 130 40
Circle -1 true false 181 129 40

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

three
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Circle -1 true false 182 180 41

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

two
false
0
Rectangle -16777216 true false 46 45 255 255
Circle -1 true false 129 130 41
Circle -1 true false 75 76 41
Circle -1 true false 185 182 38
Circle -1 true false 182 180 41
Rectangle -16777216 true false 122 120 177 175

wolf
false
0
Rectangle -7566196 true true 15 105 105 165
Rectangle -7566196 true true 45 90 105 105
Polygon -7566196 true true 60 90 83 44 104 90
Polygon -16777216 true false 67 90 82 59 97 89
Rectangle -1 true false 48 93 59 105
Rectangle -16777216 true false 51 96 55 101
Rectangle -16777216 true false 0 121 15 135
Rectangle -16777216 true false 15 136 60 151
Polygon -1 true false 15 136 23 149 31 136
Polygon -1 true false 30 151 37 136 43 151
Rectangle -7566196 true true 105 120 263 195
Rectangle -7566196 true true 108 195 259 201
Rectangle -7566196 true true 114 201 252 210
Rectangle -7566196 true true 120 210 243 214
Rectangle -7566196 true true 115 114 255 120
Rectangle -7566196 true true 128 108 248 114
Rectangle -7566196 true true 150 105 225 108
Rectangle -7566196 true true 132 214 155 270
Rectangle -7566196 true true 110 260 132 270
Rectangle -7566196 true true 210 214 232 270
Rectangle -7566196 true true 189 260 210 270
Line -7566196 true 263 127 281 155
Line -7566196 true 281 155 281 192

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0alpha1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
