turtles-own [xpos ypos zpos dz neighbor-turtles]
globals [drop-width k]

to setup
  ca
  set-default-shape turtles "circle"
  cct (screen-edge-x * screen-edge-y) [
    set ypos (floor (who / screen-edge-x))    ;Line up the turtles according to their ID
    set xpos (who - (screen-edge-x * ypos))
    set xpos (xpos - (.5 * screen-edge-x))    ;Center the resulting box
    set ypos (ypos - (.5 * screen-edge-y))
    set xcor xpos
    set ycor ypos
    set color blue
    set zpos 0
    set dz 0
  ]
  ask turtles [
    set neighbor-turtles turtles in-radius 1 with [self != myself]
  ]
  set drop-width 2
end

to go    ;Listen for mouse clicks that indicating drops, while propagating waves
  if (mouse-down?)
    [ask turtles [release-drop mouse-xcor mouse-ycor]]
  ask turtles [propagate-waves]
end

to release-drop    [drop-xpos drop-ypos]    ;Turtle procedure for releasing a drop onto the pond
  if (((xpos - drop-xpos) ^ 2) + ((ypos - drop-ypos) ^ 2) <= ((.5 * drop-width) ^ 2))
      [set dz (dz + (k * ((sum values-from neighbor-turtles [zpos]) - 
                    ((count neighbor-turtles) * zpos) - impact)))]
end

to propagate-waves    ;Turtle procedure
  set k (1 - (.01 * surface-tension))        ;k determines the degree to which neighbor-turtles' 
  set dz (dz + (k * ((sum values-from neighbor-turtles [zpos]) - ((count neighbor-turtles) * zpos))))
  set zpos ((zpos + dz) * (.01 * friction))    ;Steal energy by pulling the turtle closer
  set color scale-color blue zpos -40 40            ;to ground level
  ifelse three-D?
   [set ycor (zpos + (ypos * sin angle))
    set xcor (xpos + (ypos * cos angle))
    ifelse (abs (zpos + (ypos * sin angle)) > screen-edge-y)    ;Hide turtles above or below the screen
      [ht]
      [st]] 
   [set ycor ypos
    set xcor xpos
    ifelse (abs ypos > screen-edge-y)                ;Hide turtles above or below the screen
      [ht]
      [st]]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Raindrops model.
; http://ccl.northwestern.edu/netlogo/models/Raindrops.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Raindrops
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
321
18
636
354
30
30
5.0
1
10
1
1
1

CC-WINDOW
321
369
626
489
Command Center

SWITCH
6
198
111
231
three-D?
three-D?
1
1
-1000

BUTTON
16
37
75
70
setup
setup
NIL
1
T
OBSERVER
T

SLIDER
6
238
147
271
angle
angle
0.0
90.0
45.0
1.0
1
degrees

SLIDER
173
75
294
108
impact
impact
0.0
60.0
30.0
1.0
1
NIL

SLIDER
164
238
312
271
friction
friction
90.0
100.0
98.0
1.0
1
NIL

TEXTBOX
6
175
117
193
3D perspective

TEXTBOX
174
41
280
69
Click on pond to\nrelease a drop

SLIDER
164
204
312
237
surface-tension
surface-tension
50.0
99.0
65.0
1.0
1
NIL

TEXTBOX
175
181
287
199
Pond properties

BUTTON
79
37
139
70
go
go
T
1
T
OBSERVER
T

@#$#@#$#@
HOW TO USE IT
-------------
SETUP initializes the model.  GO continuously propagates waves in the water while listening for drops caused by the user.  To release a drop, hold your mouse button down briefly on any point within the pond (the blue region).  That point is where the drop will land.
  
The FRICTION slider controls the amount of force that pulls the water particles back towards ground level, dampening waves.  SURFACE-TENSION controls how much effect the neighbors of a water particle exert on it vertically.  The greater the surface tension, the slower waves travel through the water.  This is implemented in the model as a weight coefficient on the difference in height between a turtle and its neighbors.  The resulting weighted value is added to the turtles current vertical velocity (represented by the incremental value dz) and its current position to determine its next position.  To translate the earlier statement, then, the greater the surface tension, the more a particle's neighbors can pull it towards them, thus quickening the pace of a wave through the surface.

The IMPACT slider determines how much force a drop carries.  Drops with greater impact cause larger waves.

The THREE-D switch allows you to view the pond either from above or from an angle set by ANGLE.


RUNNING THE MODEL
-----------------
Click the SETUP button to set up the pond, then GO to enable motion in the water and listen for drops.  Periodically hold the mouse button down over the pond to release raindrops and initiate waves.


THINGS TO NOTICE
----------------
The pond is made up of a grid of turtles, each of whom, like in Wave Machine, behaves like it is connected to its neighbors by springs.  Each turtle has a vertical position and velocity, which determine how high above the sides of the pond it is or how far below, and how much that distance will change at the next time step.   

When drops are released they decrease the vertical velocity of the particles under them, causing them to move downwards.  As they fall, they pull down their neighbors, who at the same time pull them up. This exchange creates a wave within the pond, centered on where the drop fell.

As the waves hit the walls, they are reflected back towards the middle, as the walls don't move up or down.  Friction causes the waves to eventually decline by pulling them closer to the surface level.


THINGS TO TRY
-------------
Try creating multiple drops in the pool to see how they interact.  Do they cancel each other out or reinforce each other?  

Change the surface tension to see how it affects the speed of waves.  Why does water with low surface tension move so slowly?  What sort of liquids in real life is it similar to?

Set friction to 100 and release some drops.  Do the waves ever dissipate?  How can the waves rise above the sides of the pond (have positive zpos's) when only downward moving forces are exerted on them?  How is a turtle's movement similar to that of a spring.  Is the friction used in this model analogous to that in a spring?   


EXTENDING THE MODEL
-------------------
This model doesn't take into account water pressure.  When part of the water is pushed down on the impact of a drop, the rest of the water should feel a push from beneath.  Try adding in this mechanism.

The walls in this model don't affect waves at all.  Extend the model so that the walls impose friction on waves that brush against them, causing them to dampen.


NETLOGO FEATURES
------------------
In order to create the visual impression of a wave, it is necessary for water above the sides of the pond (above water level) to appear more white, while water below to appear more blue.  This was accomplished using the "scale-color" primitive, which takes a color to assign, a variable, and a minimum and maximum value.  It works by comparing the value of the variable for each turtle to the two values, and the closer it is to the maximum, the lighter the shade of the color assigned to that turtle.

Water particles that fly off the top or bottom of the screen are ignored in the model, which is done by using the commands hide-turtle (ht) and show-turtles (st).  Hide turtle doesn't cause the turtle to die or change in any way, but merely stops it from being drawn on the screen.  Show turtle causes it to begin being displayed again. 

The model listens for raindrops using NetLogo's mouse primitives.  GO repeatedly calls mouse-down? to register when the user has clicked over the pond.  When this condition reports true, it uses two reporters, mouse-xcor and mouse-ycor, to record the current coordinates of the mouse so that a drop can be released in the appropriate place.  


RELATED MODELS
--------------
For another model that demonstrates how turtles behave as a surface, see Wave Machine.  In it, turtles are connected in the same manner as outlined above, but their movement is governed by the regular sinusoidal motion of part of the surface.


CREDITS AND MODELS
------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Raindrops model. http://ccl.northwestern.edu/netlogo/models/Raindrops. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Raindrops for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

circle
true
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

@#$#@#$#@
NetLogo 2.0beta4
@#$#@#$#@
set three-D? true
setup
go
ask turtles [ release-drop 3 3 ]
repeat 20 [ go ]
@#$#@#$#@
@#$#@#$#@
