;; The procedures for this model have been kept largely intact from
;; the original code written by the student. With advances in the
;; language, this code is no longer at all an optimal way of writing
;; this model. We have kept the original code for research purposes --
;; please do not use it as an example of good NetLogo coding.

turtles-own [ random-no total-ratio step x-zoom x-ave ] 
patches-own [ turtle-quant turtle-bottom ] 

globals  [
  x-ave-glob
  x-scale kx
  y-scale ky
  y-scale-distr ky-distr
  step1 step2 step3 step4 step5 steps total-hops
  ratio1 ratio2 ratio3 ratio4 ratio5 ratios total-ratios
]

;; x-zoom: intermediate variable for zooming
;; x-ave: intermediate variable for average line
;; x-ave-glob: intermediate variable for average line 
;; kx: x axis scaling factor = 2 ^ x_scale
;; ky: normalization factor for the original line
;; ky-distr: normalization factor for the distribution line
;; step1 to step5: step sizes 
;; total-hops: total number of hops for the current setup
;; ratio1 to ratio5: probability ratio corresponding to steps
;; turtle-quant: number of turtles on each patch
;; turtle-bottom: number of turtles on each patch of the bottom line


to setup
  ca  
  set steps sentence hop-1 list hop-2 hop-3
  set ratios sentence ratio-1 list ratio-2 ratio-3 
  cct number [
    set x-zoom  0  
    set x-ave  0   
    setxy 0  (- screen-edge-x) 
    set color red
    set shape  "bunny"
    set heading  90 
  ]  
  set kx  x-scaling
  
;; below is the y-scale factor to ensure that the height of the original line and
;; average line are always about the same height, 90% of the screen height.  
  set ky  (number / (screen-size-y - 1)) / 0.9  
  set y-scale-distr  (screen-size-x - 1) * ky  
  set y-scale  0  
  set x-scale  (screen-size-x - 1) * kx
  set ky-distr  0
  set total-hops  0
  ask patches [ plot-green ]
  define-steps
  define-ratios
end

to go-one-hop 
  hopping  
  plot-violet  
  plot-yellow  
  set total-hops  total-hops + 1  
end

to go
   if hops = total-hops [ stop ]
   go-one-hop
end

to hopping 
  set kx  x-scaling 
  set x-scale  (screen-size-x - 1) * kx 
  ask turtles  [
    ht 
    set-step 
    set x-zoom  (x-zoom + step) 
    set x-ave  (x-zoom / ky) 
    setxy (x-zoom / kx)  ycor 
    st
  ]
end

to set-step  ;; turtle procedure
  set total-ratio  (ratio1 + ratio2 + ratio3 + ratio4 + ratio5)
  set random-no  (random-float total-ratio) 
  if random-no <= ratio1 
    [ set step step1 ] 
  if (random-no > ratio1) and (random-no <= ratio1 + ratio2) 
    [ set step step2 ] 
  if (random-no > ratio1 + ratio2) and (random-no <= ratio1 + ratio2 + ratio3) 
    [ set step step3 ] 
  if (random-no > ratio1 + ratio2 + ratio3) 
     and (random-no <= ratio1 + ratio2 + ratio3 + ratio4) 
    [ set step step4 ] 
  if random-no > ratio1 + ratio2 + ratio3 + ratio4 
    [ set step step5 ] 
end

to plot-yellow 
  locals [ most-turtles ]
  ask patches  [ 
    unplot-yellow 
    set turtle-quant (count turtles-here)
  ]
  
  set most-turtles  max values-from patches [ turtle-quant ]
  ask patches [ 
    set ky-distr  (most-turtles / ky / int (.80 * screen-size-y)) 
    set y-scale  (100 * ky-distr * ky) 
    set turtle-bottom  (((turtle-quant-of patch-at 0 ((- pycor) - screen-edge-y)) 
                         / ky) / ky-distr) 
    if (pycor + screen-edge-y) < turtle-bottom  
      [ set pcolor yellow ] 
  ]
end

to plot-violet 
  ask patches [ unplot-violet ]
  set x-ave-glob  ((( sum values-from turtles [ x-ave ]) * ky) / number)   
  ask patches [ 
   if   ((pxcor = round  (x-ave-glob / kx))   
         or  (pxcor = (round  ((x-ave-glob / kx) - screen-size-x))) 
         or  (pxcor = (round  ((x-ave-glob / kx) - (2 * screen-size-x)))))  
        and  (pycor <= ((number / ky) - screen-edge-y))
       [ set pcolor  violet ]  
  ]
end

to plot-green   ;; patch procedure
  if (pxcor = 0) and (pycor <= ((number / ky) - screen-edge-y)) 
  [ set pcolor green ] 
end

to define-steps 
  if not empty? steps  [ 
    set step1 first steps 
    set steps butfirst steps 
  ] 
  if not empty? steps  [ 
    set step2 first steps 
    set steps butfirst steps 
  ] 
  if not empty? steps  [ 
     set step3 first steps 
    set steps butfirst steps 
  ] 
  if not empty? steps  [ 
    set step4 first steps 
    set steps butfirst steps 
  ] 
  if not empty? steps  [ 
    set step5 first steps 
    set steps butfirst steps 
  ] 
end

to define-ratios 
  set total-ratios 0 
  if not empty? ratios  [ 
    set ratio1 first ratios 
    set ratios butfirst ratios 
  ] 
  if not empty? ratios  [ 
    set ratio2 first ratios 
    set ratios butfirst ratios 
  ] 
  if not empty? ratios  [ 
    set ratio3 first ratios 
    set ratios butfirst ratios 
  ] 
  if not empty? ratios  [ 
    set ratio4 first ratios 
    set ratios butfirst ratios 
  ] 
  if not empty? ratios  [ 
    set ratio5 first ratios 
    set ratios butfirst ratios 
  ] 
end

to unplot-yellow   ;; patch procedure
  if pcolor = yellow 
    [ set pcolor black ] 
end

to unplot-violet   ;; patch procedure
  if pcolor = violet 
    [ set pcolor black ] 
  plot-green
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Binomial Rabbits model.
; http://ccl.northwestern.edu/netlogo/models/BinomialRabbits.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/BinomialRabbits
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
282
10
697
446
22
22
9.0
1
10
1
1
1

CC-WINDOW
2
344
279
501
Command Center

BUTTON
193
111
281
144
go
go
T
1
T
OBSERVER

BUTTON
193
42
281
76
setup
setup
NIL
1
T
OBSERVER

BUTTON
193
76
281
111
go-one-hop
go-one-hop
NIL
1
T
OBSERVER

SLIDER
3
42
95
75
hop-1
hop-1
-5
5
1
1
1
NIL

SLIDER
3
80
95
113
hop-2
hop-2
-5
5
-1
1
1
NIL

SLIDER
3
119
95
152
hop-3
hop-3
-5
5
0
1
1
NIL

SLIDER
95
42
187
75
ratio-1
ratio-1
0.0
10.0
1.0
1.0
1
NIL

SLIDER
95
80
187
113
ratio-2
ratio-2
0.0
10.0
1.0
1.0
1
NIL

SLIDER
3
199
187
232
hops
hops
0
99
20
1
1
NIL

SLIDER
74
251
176
284
x-scaling
x-scaling
1.0
10.0
1.0
1.0
1
NIL

SLIDER
3
166
187
199
number
number
1
4000
2400
1
1
NIL

SLIDER
95
119
187
152
ratio-3
ratio-3
0.0
10.0
0.0
1.0
1
NIL

MONITOR
3
240
67
289
x-scale
x-scale
0
1

MONITOR
3
292
67
341
y-scale
y-scale
0
1

MONITOR
206
292
279
341
total-hops
total-hops
0
1

TEXTBOX
370
453
680
507
GREEN LINE - Starting rabbit position\nPURPLE LINE - Average rabbit position\nYELLOW LINES - Number of rabbits at that position

@#$#@#$#@
WHAT IS IT?
-----------
This model simulates a binomial probability distribution or (in the limit) normal distribution.

This model was created by a student in an effort to make sense of normal distributions.  In particular, he sought to understand  why height is distributed normally in human populations. For a detailed account of this case, see: Wilensky, U. (1997). What is Normal Anyway? Therapy for Epistemological Anxiety.  Educational Studies in Mathematics. Volume 33, No. 2. pp. 171-202. http://ccl.northwestern.edu/cm/papers/normal/.

The procedures for this model have been kept largely intact from the original code written by the student. With advances in the language, this code is no longer at all an optimal way of writing this model. We have kept the original code for research purposes -- please do not use it as an example of good NetLogo coding.

The model works by analogizing height variations to rabbit hops.

A number of rabbits are placed at the center of the bottom of the graphics window.  A move pattern determines the way a rabbit moves.  Each rabbit can choose to hop right or left a certain hop-size. The likelihood of a rabbit following each move pattern is given in terms of ratios.  Each rabbit may have up to five different move patterns.


HOW TO USE IT?
--------------
Setup

Method one (sliders setup): Press SETUP button. This creates the number of rabbits from the NUMBER slider and up to three hops and associated probability ratios from the six sliders above the NUMBER slider.  Each time a rabbit hops, it chooses one of the three moves -- hop-1, hop-2, or hop-3 -- with a likelihood in the ratio of ratio-1, ratio-2, and ratio-3 to each other.  For example, if ratio-1 = 2, ratio-2 = 4, and ratio-3 = 6, the rabbit has a 2-in-12 chance of making the hop-1 move, a 4-in-12 chance of making the hop-2 move, and a 6-in-12 chance of making the hop-3 move.  

Method two (manual setup): In the Command Center, type "setup [number] [list of hops [list of probability ratios]" to initialize the rabbits (e.g. "setup 4000 [1 -1] [1 2]" will set up 4000 rabbits hopping either one unit to the right(1) or one unit to the left (-1) with a chance of hopping to the left being twice as much as that to the right.)  Up to five steps and corresponding probability ratios can be used.

The GO-ONE-HOP button makes each rabbit hop once.

The GO button tells the rabbits to hop the number of times set by the HOPS slider. For example, if HOPS is set to 10, the GO button makes each rabbit hop 10 times.  To stop the rabbits from hopping once they've started, press the GO button again.

There are two scale monitors and one scale slider in the Interface Window.  X-SCALING is used to magnify the width of the screen to facilitate more hops. It is manually set by users with the X-SCALING slider. The setting can be changed as the model runs.  Y-SCALE is used to regulate the vertical scale -- to ensure that the highest yellow distribution bar is always 80% of the height of the screen. This is done at each hop.

The figure inside the "y-scale" monitor is the number of rabbits a screen-size-long yellow line represents.  The figure inside the "x-scale" monitor is the number of steps represented by a full screen. (The rabbits wrap the sceen, so if they get to the edge, you should increase the x-scale.)

The following formulae can be used to evaluate the actual numbers of rabbits or steps hopped:

  Actual Number of Rabbits for a Yellow Line =
  height of line * ( y-scale / 100 )
  (To find out exactly how many rabbits are represented by a line, cntl-click (mac) or right-click(other) anywhere on the line and choose inspect patch from the menu that appears.  The inspector will have a variable "turtle-bottom" which will tell you how many turtles(rabbits) are at the bottom of the line.)

  Cumulative Number of Steps Hopped so far =
  X-coordinate of a line * ( x-scale / 100 )


THINGS TO NOTICE
----------------
The purple average line shows where an average rabbit would be. Observe the movement of this line -- both its position and velocity -- and try to relate these to the settings. 

Play with the NUMBER slider to see if what you predict is what you see when the number of rabbits is small. For what numbers of rabbits are your predictions the most accurate?


THINGS TO TRY
-------------
Try different values for list of steps. What happens to the distribution?

Try different values for probability ratios.  What happens to the distribution?

Is the distribution always symmetric? What would you expect?


EXTENDING THE MODEL
-------------------
Create a plot for 'hopping'. First decide what to plot, and then implement the proper NetLogo plot functions.
Rewrite the model so rabbits take list variables. Are there now new capabilities you can give the rabbits?


NETLOGO FEATURES
----------------
The limitation on the number of turtles constrains the limits of the "number" slider. You can make the corresponding change to the NUMBER slider - select the slider by clicking and dragging the mouse button over it.  Then click on the edit button and change 'Maximum' to the new number. Having more rabbits to jump can be useful for certain statistical simulations.

You can also change the settings to have a bigger graphics window to fit more hops or show very fine distribution diagrams.

Note that since turtles could not have list variables in earlier versions of the language, the global lists steps and ratios are used to hold the movement patterns and ratios. The turtles access these globals to know how to move. (if we were writing this model now, we would not code it this way as turtles in NetLogo can have list variables). The procedures 'define-steps' and 'define-ratios' use the primitives 'first' and 'butfirst'. Both of these are list operators - that is, they operate on lists of things. The 'first' of a list is simply its first element. Likewise, the 'butfirst' of a list is a list of all elements except for the first.


RELATED MODELS
--------------
Galton Box, Random Walk Left Right


CREDITS AND REFERENCES
----------------------
See: Wilensky, U. (1997). What is Normal Anyway? Therapy for Epistemological Anxiety.  Educational Studies in Mathematics. Volume 33, No. 2. pp. 171-202. http://ccl.northwestern.edu/cm/papers/normal/

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Binomial Rabbits model. http://ccl.northwestern.edu/netlogo/models/BinomialRabbits. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/BinomialRabbits for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 152 149 77 163 67 195 67 211 74 234 85 252 100 264 116 276 134 286 151 300 167 285 182 278 206 260 220 242 226 218 226 195 222 166
Polygon -16777216 true false 150 149 128 151 114 151 98 145 80 122 80 103 81 83 95 67 117 58 141 54 151 53 177 55 195 66 207 82 211 94 211 116 204 139 189 149 171 152
Polygon -7566196 true true 151 54 119 59 96 60 81 50 78 39 87 25 103 18 115 23 121 13 150 1 180 14 189 23 197 17 210 19 222 30 222 44 212 57 192 58
Polygon -16777216 true false 70 185 74 171 223 172 224 186
Polygon -16777216 true false 67 211 71 226 224 226 225 211 67 211
Polygon -16777216 true false 91 257 106 269 195 269 211 255
Line -1 false 144 100 70 87
Line -1 false 70 87 45 87
Line -1 false 45 86 26 97
Line -1 false 26 96 22 115
Line -1 false 22 115 25 130
Line -1 false 26 131 37 141
Line -1 false 37 141 55 144
Line -1 false 55 143 143 101
Line -1 false 141 100 227 138
Line -1 false 227 138 241 137
Line -1 false 241 137 249 129
Line -1 false 249 129 254 110
Line -1 false 253 108 248 97
Line -1 false 249 95 235 82
Line -1 false 235 82 144 100

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

bunny
false
0
Circle -65536 true false 67 127 171
Polygon -65536 true false 87 157 18 66 10 27 25 5 54 5 77 24 111 147 105 170
Polygon -65536 true false 192 140 205 37 220 10 248 2 274 7 283 29 279 50 256 77 209 151 185 167

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 2.0alpha1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
